@@ -1,6 +1,7 @@
-from aquests.lib.athreads import trigger
+from aquests.athreads import trigger
 from aquests import lifetime
-import sys, asyncore, time
+import sys, time
+from rs4 import asyncore
 import gc
 import select
 import os
@@ -8,7 +9,6 @@ import bisect
 import socket
 import time
 
-
 if os.name == "nt":
 	from errno import WSAENOTSOCK
 	
@@ -23,39 +23,33 @@ _select_errors = 0
 def status ():
 	fds = []
 	for fdno, channel in list(asyncore.socket_map.items ()):
-		try:
-			d = {}
-			d ["name"] = "%s.%s" % (channel.__module__, channel.__class__.__name__)
-			d ["fdno"] = fdno
-			
-			status = "NOTCON"
-			if channel.accepting and channel.addr: status = "LISTEN"
-			elif channel.connected: status = "CONNECTED"
-			d ["status"] = status
-			
-			addr = ""
-			if channel.addr is not None:
-				try: addr = "%s:%d" % channel.addr
-				except TypeError: addr = "%s" % repr (channel.addr)
-			if addr:
-				d ["address"] = addr			
-			if hasattr (channel, "channel_number"):
-				d ["channel_number"] = channel.channel_number
-			if hasattr (channel, "request_counter"):	
-				d ["request_counter"] = channel.request_counter
-			if hasattr (channel, "event_time"):
-				d ["last_event_time"] = time.asctime (time.localtime (channel.event_time))
-			if hasattr (channel, "zombie_timeout"):
-				d ["zombie_timeout"] = channel.zombie_timeout
-			if hasattr (channel, "get_history"):
-				d ["history"] = channel.get_history ()
-							
-		except:
-			raise
-			pass
+		d = {}	
+		d ["name"] = "%s.%s" % (channel.__module__, channel.__class__.__name__)
+		d ["fdno"] = fdno
 		
-		else:		
-			fds.append (d)
+		status = "NOTCON"
+		if channel.accepting and channel.addr: status = "LISTEN"
+		elif channel.connected: status = "CONNECTED"
+		d ["status"] = status		
+		addr = ""
+		if channel.addr is not None:
+			try: addr = "%s:%d" % channel.addr
+			except TypeError: addr = "%s" % repr (channel.addr)
+		if addr:
+			d ["address"] = addr
+
+		if hasattr (channel, "channel_number"):
+			d ["channel_number"] = channel.channel_number
+		if hasattr (channel, "request_counter"):	
+			d ["request_counter"] = channel.request_counter
+		if hasattr (channel, "event_time"):
+			d ["last_event_time"] = time.asctime (time.localtime (channel.event_time))
+		if hasattr (channel, "zombie_timeout"):
+			d ["zombie_timeout"] = channel.zombie_timeout
+		if hasattr (channel, "get_history"):
+			d ["history"] = channel.get_history ()
+						
+		fds.append (d)
 		
 	return {
 		"killed_zombies": _killed_zombies,
@@ -80,13 +74,32 @@ def maintern_zombie_channel (now):
 				except:
 					channel.handle_error ()
 
-maintern = None
-def init (kill_zombie_interval = 10.0):
-	global maintern
-	maintern = lifetime.Maintern ()
+maintern = lifetime.Maintern ()
+def init (kill_zombie_interval = 60.0, logger = None):
+	global maintern	
+	lifetime.EXHAUST_DNS = False
+	lifetime._logger = logger
 	maintern.sched (kill_zombie_interval, lifetime.maintern_zombie_channel)
-	maintern.sched (300.0, lifetime.maintern_gc)
+	maintern.sched (60.0, maintern_gc)
+	if "---gc" in sys.argv:
+		gc.set_debug (gc.DEBUG_SAVEALL)	
 
+def maintern_gc (now):
+	gc.collect ()
+	if "---gc" in sys.argv:
+		print ("# Garbages collected ===========================")
+		for item in gc.garbage:
+			print ("  -", item)	
+	
+def manual_gc (interval = 60.0):
+	global maintern
+	maintern.sched (interval, maintern_gc)
+	gc.disable ()
+
+def enable_memory_track (interval = 10.0):
+	global maintern		
+	maintern.sched (interval, lifetime.summary_track)	
+	
 def shutdown (exit_code, shutdown_timeout = 30.0):
 	global _shutdown_phase
 	global _shutdown_timeout
@@ -110,7 +123,7 @@ def loop (timeout = 30.0):
 	global _exit_code
 	global maintern
 	
-	if maintern is None:
+	if not maintern.q:
 		init ()
 		
 	_shutdown_phase = 0
@@ -120,22 +133,23 @@ def loop (timeout = 30.0):
 	try: 
 		lifetime_loop(timeout)
 	except KeyboardInterrupt:
+		_shutdown_timeout = 1
 		graceful_shutdown_loop()
 	else:
-		graceful_shutdown_loop()	
+		graceful_shutdown_loop()
 
-def lifetime_loop (timeout = 30.0):
+def lifetime_loop (timeout = 30.0, map = None):
 	global _last_maintern
 	global _maintern_interval
 				
-	map = asyncore.socket_map
+	map = map or asyncore.socket_map
 	while map and _shutdown_phase == 0:
 		lifetime.poll_fun_wrap (timeout, map)
 		now = time.time()
 		if (now - _last_maintern) > _maintern_interval:
-			maintern (now)
+			maintern and maintern (now)
 			_last_maintern = time.time ()
-		
+				
 def graceful_shutdown_loop ():
 	global _shutdown_phase
 	timestamp = time.time()
@@ -160,7 +174,3 @@ def graceful_shutdown_loop ():
 		else:
 			_shutdown_phase += 1
 			timestamp = time.time()
-
-
-
-
